from helios.webapp import base_app
import re
import json

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin


# This script detects vulnerabilities in the following PHP based products:
# - Typo 3
class Scanner(base_app.BaseAPP):

    def __init__(self):
        self.name = "Typo3"
        self.types = []

    def detect(self, url):
        directories = ['', 'typo3']
        for d in directories:
            path = urljoin(url, d)
            response = self.send(path)
            if response and response.status_code == 200 and \
                    ("/typo3" in response.text or 'generator" content="TYPO3' in response.text):
                self.app_url = path
                return True
        return False

    def test(self, url):
        root_page = self.send(url)
        version = None
        if root_page:
            get_version = re.search(r'<meta name="generator" content="typo3 ([\d\.]+)', root_page.text, re.IGNORECASE)
            if get_version:
                version = get_version.group(1)
                self.logger.info("%s version %s was identified from the generator tag" % (self.name, version))
        if not version:
            cl_url = urljoin(url, 'ChangeLog')
            changelog_page = self.send(cl_url)
            version = None
            if changelog_page and changelog_page.url == cl_url:
                get_version = re.search(r'Release of TYPO3 ([\d\.]+)', changelog_page.text, re.IGNORECASE)
                if get_version:
                    version = get_version.group(1)
                    self.logger.info("%s version %s was identified from the ChangeLog file" % (self.name, version))

        if version:
            db = self.get_db("typo3_vulns.json")
            data = json.loads(db)
            vulns = data['Typo3']
            self.match_versions(vulns, version, url)



import requests
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse
import helios.modules.module_base
from helios.core.utils import requests_response_to_dict


class Module(helios.modules.module_base.Base):

    def __init__(self):
        self.name = "Shellshock"
        self.input = "urls"
        self.injections = {}
        self.module_types = ['injection']
        self.possibilities = [
            '.cgi'
        ]
        self.output = "vulns"
        self.severity = 3

    def run(self, urls, headers={}, cookies={}):
        results = []
        list_urls = []
        self.cookies = cookies
        for u in urls:
            if u[0].split('?')[0] not in list_urls:
                list_urls.append(u[0].split('?')[0])
        for f in list_urls:
            for p in self.possibilities:
                if p in f:
                    result = self.test(f)
                    if result:
                        results.append(result)
        return results

    def test(self, url):
        payload = {
            'User-Agent': '() { ignored;};id ',
            'Referer': '() { ignored;};id '
        }
        result = self.send(url, headers=payload)
        if result and 'gid=' in result.text and 'uid=' in result.text:
            result_obj = {'request': requests_response_to_dict(result), "match": "Command was executed on server"}
            return result_obj
        return None

    def send(self, url, headers={}):
        result = None
        cookies = self.cookies
        try:
            result = requests.get(url, headers=headers, cookies=cookies)
        except Exception as e:
           pass
        return result



from helios.webapp import base_app
import re
import json

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin


# This script detects vulnerabilities in the following PHP based products:
# - MODX Revolution
class Scanner(base_app.BaseAPP):

    def __init__(self):
        self.name = "MODX Revolution"
        self.types = []

    def detect(self, url):
        directories = ['', 'manager']
        for d in directories:
            path = urljoin(url, d)
            response = self.send(path)
            if response and response.status_code == 200:

                if ('<section class="rst-sidemenu' in response.text or
                     '<nav class="rst-topmenu' in response.text):
                    self.app_url = path
                    return True

                if d == "manager" and 'core/modx.js' in response.text:
                    path = urljoin(url, '')
                    self.app_url = path
                    return True
        return False

    def test(self, url):
        # bit of a long shot since there are only limited demo sites available to test on :)
        revo = urljoin(url, 'core/docs/changelog.txt')
        revo_page = self.send(revo)
        version = None
        if revo_page:
            # only (accurate) way to get installed version (ignore the -p1..etc parts since they seem to change a lot)
            get_version = re.search(r'MODX Revolution\s([\d\.]+-\w+)', revo_page.text, re.IGNORECASE)
            if get_version:
                version = get_version.group(1)
                self.logger.info("%s version %s was identified from the core/docs/changelog.txt file" %
                                 (self.name, version))

        if version:
            db = self.get_db("modx_revolution_vulns.json")
            data = json.loads(db)
            vulns = data['Revolution']
            self.match_versions(vulns, version, url)



import sys
import os
import re
import json

import cms_scanner

try:
    from urlparse import urljoin, urlparse
except ImportError:
    from urllib.parse import urljoin, urlparse


class Scanner(cms_scanner.Scanner):
    plugin_update_url = "https://data.wpscan.org/plugins.json"
    plugin_cache_file = "wordpress_plugins.json"

    version_update_url = "https://data.wpscan.org/wordpresses.json"
    version_cache_file = "wordpress_versions.json"

    def __init__(self):
        self.name = "wordpress"
        self.update_frequency = 3600 * 48

    def get_version(self, url):
        text = self.get(url)
        if text:
            version_check_1 = re.search(r'<meta name="generator" content="wordpress ([\d\.]+)', text.text, re.IGNORECASE)
            if version_check_1:
                self.logger.info("Detected %s version %s on %s trough generator tag" % (self.name, version_check_1.group(1), url))
                return version_check_1.group(1)
        check2_url = urljoin(url, 'wp-admin.php')
        text = self.get(check2_url)
        if text:
            version_check_2 = re.search(r'wp-admin\.min\.css\?ver=([\d\.]+)', text.text, re.IGNORECASE)
            if version_check_2:
                self.logger.info("Detected %s version %s on %s trough admin css" % (self.name, version_check_2.group(1), url))
                return version_check_2.group(1)
        self.logger.warning("Unable to detect %s version on url %s" % (self.name, url))

    def run(self, base):
        self.set_logger()
        self.setup()

        version = self.get_version(base)
        info = self.get_version_info(version)

        disco = self.sub_discovery(base)

        plugins = self.read_plugins()
        plugin_data = {}
        vulns = {}
        self.logger.debug("Checking %s for %d plugins" % (base, len(plugins)))
        for plugin in plugins:
            plugin_version = self.get_plugin_version(base, plugin)
            if plugin_version:
                plugin_data[plugin] = plugin_version
                self.logger.debug("Got %s as version number for plugin %s" % (plugin_version, plugin))
                p_vulns = self.get_vulns(plugin_version, plugins[plugin])
                self.logger.info("Found %d vulns for plugin %s version %s" % (len(p_vulns), plugin, version))
                vulns[plugin] = p_vulns
        return {
            'version': version,
            'plugins': plugin_data,
            'plugin_vulns': vulns,
            'version_vulns': info,
            'discovery': disco
        }

    def get_vulns(self, version, plugin):
        vulns = []
        for vuln in plugin['vulnerabilities']:
            if self.match_versions(version, vuln['fixed_in']):
                vulns.append(vuln)
        return vulns

    def get_version_info(self, version):
        data = ""
        with open(self.cache_file(self.version_cache_file), 'rb') as f:
            data = f.read().strip()
        json_data = json.loads(data)
        if version in json_data:
            return json_data[version]
        return None

    def sub_discovery(self, base):
        logins = []
        get_users = self.get(urljoin(base, 'wp-json/wp/v2/users'))
        if get_users:
            result = json.loads(get_users.text)
            for user in result:
                user_id = user['id']
                name = user['name']
                login = user['slug']
                logins.append({'id': user_id, 'username': login, 'name': name})

        return {'users': logins}

    def read_plugins(self):
        try:
            plugins = {}
            with open(self.cache_file(self.plugin_cache_file), 'rb') as f:
                data = f.read().strip()
            json_data = json.loads(data)
            for x in json_data:
                if json_data[x]['popular'] or self.aggressive:
                    plugins[x] = json_data[x]
            return plugins
        except Exception as e:
            self.logger.error("Error reading database file, cannot init plugin database: %s" % str(e))
            return {}

    def get_plugin_version(self, base, plugin):
        plugin_readme_url = urljoin(base, 'wp-content/plugins/%s/readme.txt' % plugin)
        get_result = self.get(plugin_readme_url)
        if get_result and get_result.status_code == 200:
            self.logger.debug("Plugin %s exists, getting version from readme.txt" % plugin)
            text = get_result.text
            get_version = re.search(r'(?s)changelog.+?(\d+\.\d+(?:\.\d+)?)', text, re.IGNORECASE)
            if get_version:
                return get_version.group(1)
        return None

    def update(self):
        try:
            self.logger.info("Updating plugin files")
            data1 = self.get(self.plugin_update_url)
            with open(self.cache_file(self.plugin_cache_file), 'wb') as f:
                x = data1.content
                f.write(x)
            self.logger.info("Updating version files")
            data2 = self.get(self.version_update_url)
            with open(self.cache_file(self.version_cache_file), 'wb') as f:
                x = data2.content
                f.write(x)
            self.logger.info("Update complete")
        except Exception as e:
            self.logger.error("Error updating databases: %s" % str(e))
        return



import json
import requests
import re
import logging
import os


# some code to spider CVE entries (+ vulnerable versions) from cvedetails.com
class CVEProcessor:
    pool = {

    }

    vuln_versions = {

    }
    logger = logging.getLogger("Updater")

    def parse_cve(self, name, text):
        seen = []
        for cve in re.findall(r'(CVE-\d+-\d+) security vulnerability details', text):
            seen.append(cve)

        if name in self.pool:
            self.pool[name].extend(seen)
        else:
            self.pool[name] = seen
        return seen

    def get_versions(self, name, cve):
        if name not in self.vuln_versions:
            self.vuln_versions[name] = {}
        url = "https://www.cvedetails.com/cve/%s/" % cve
        result = requests.get(url)
        text = result.text
        splitter = text.split('<table class="listtable" id="vulnprodstable">')[1]
        splitter = splitter.split('</table>')[0]
        versions = []
        for x in re.findall(r'(?s)(<tr.+?</tr>)', splitter)[1:]:
            version = re.findall(r'(?s)<td>(.+?)</td>', x)[3].strip()
            if version not in versions:
                versions.append(version)
        self.vuln_versions[name][cve] = versions
        return versions

    def get_cve_pages(self, name, start_url):
        url = start_url
        while 1:
            result = requests.get(url)
            found = self.parse_cve(name, result.text)
            self.logger.debug("Found %d new CVE entries on page" % len(found))
            next_page = re.search(r'\(This Page\)\s*<a.+?href="(.+?)"', result.text)
            if next_page:
                url = "https://www.cvedetails.com%s" % next_page.group(1)
                self.logger.debug("Next page: %s" % url)
            else:
                break

    def run(self, entries, output):
        for name in entries:
            self.get_cve_pages(name, entries[name])
            for item in self.pool[name]:
                vs = self.get_versions(name, item)
                self.logger.debug("%d vuln versions of %s for %s" % (len(vs), name, item))
        output_file = '%s_vulns.json' % output
        with open(os.path.join(os.path.dirname(__file__), output_file), 'w') as f:
            self.logger.info("Writing CVE/Version combo to output file %s" % output)
            f.write(json.dumps(self.vuln_versions))



from helios.webapp import base_app
import re
import json

try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin


# This script detects vulnerabilities in the following PHP based products:
# - Concrete5
class Scanner(base_app.BaseAPP):

    def __init__(self):
        self.name = "Concrete5"
        self.types = []

    def detect(self, url):
        directories = ['', 'blog', 'admin']
        for d in directories:
            path = urljoin(url, d)
            response = self.send(path)
            if response and response.status_code == 200:
                if 'concrete5' in response.text:
                    self.app_url = path
                    return True
        return False

    def test(self, url):
        root = urljoin(url, '')
        root_page = self.send(root)
        version = None
        if root_page:
            get_version = re.search(r'<meta name="generator" content="concrete5 - ([\d\.]+)',
                                    root_page.text, re.IGNORECASE)
            if get_version:
                version = get_version.group(1)
                self.logger.info("%s version %s was identified from the generator tag" %
                                 (self.name, version))

        if version:
            db = self.get_db("concrete5_vulns.json")
            data = json.loads(db)
            vulns = data['Concrete5']
            self.match_versions(vulns, version, url)



from setuptools import setup
from os.path import dirname, join, abspath

setup(
    name='Helios Scanner',
    version='1.1',
    packages=['helios', 'helios.ext', 'helios.ext.libcms', 'helios.ext.libcms.scanners', 'helios.ext.mefjus',
              'helios.ext.metamonster', 'helios.core', 'helios.webapp', 'helios.webapp.databases', 'helios.modules'],
    url='https://github.com/stefan2200/Helios',
    license='Apache 2.0',
    author='stefan2200',
    author_email='stefan@stefanvlems.nl',
    description='A Python based Web Application security scanner',
    long_description=open(join(abspath(dirname(__file__)), "README.rst")).read(),
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS :: MacOS X',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Security',
        'Topic :: Software Development :: Testing',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    entry_points={
        'console_scripts': ['helios=helios.main:main', 'helios-update-db=helios.webapp.databases.update:main'],
    },
    install_requires=[
        'pyOpenSSL>=0.14'
        'beautifulsoup4',
        'requests',
        'selenium',
        'filelock',
        'msgpack'
    ],
    extras_require={':python_version == "2.7"': ['futures']},
    include_package_data=True,
    package_data={
        '': [
            'scripts/*',
            'drivers/*',
            'wordlists/*'
        ]
    }
)



import optparse
import sys
import logging

import helios.webapp.databases.process as process

config = {
    'typo3': {
        'Typo3': 'https://www.cvedetails.com/vulnerability-list/vendor_id-3887/Typo3.html'
    },
    'tomcat': {
        'Tomcat': 'https://www.cvedetails.com/vulnerability-list/vendor_id-45/product_id-887/Apache-Tomcat.html',
        'Jboss': 'https://www.cvedetails.com/vulnerability-list/vendor_id-25/product_id-14972/Redhat-Jboss-Enterprise-Application-Platform.html'
    },
    'subrion': {
        'Subrion': 'https://www.cvedetails.com/vulnerability-list/vendor_id-9328/product_id-23428/Intelliants-Subrion-Cms.html'
    },
    'textpattern': {
        'Textpattern': 'https://www.cvedetails.com/vulnerability-list/vendor_id-5344/Textpattern.html'
    },
    'cmsmadesimple': {
        'CmsMadeSimple': 'https://www.cvedetails.com/vulnerability-list/vendor_id-3206/Cmsmadesimple.html'
    },
    'magento': {
        'Magento': 'https://www.cvedetails.com/vulnerability-list/vendor_id-15393/product_id-31613/Magento-Magento.html'
    },
    'modx_revolution': {
        'Revolution': 'https://www.cvedetails.com/vulnerability-list/vendor_id-11576/product_id-23360/Modx-Modx-Revolution.html'
    },
    'phpmyadmin': {
        'phpMyAdmin': 'https://www.cvedetails.com/vulnerability-list/vendor_id-784/Phpmyadmin.html'
    },
    'concrete5': {
        'Concrete5': 'https://www.cvedetails.com/vulnerability-list/vendor_id-11506/product_id-23747/Concrete5-Concrete5.html'
    },
    'php': {
        'PHP': 'https://www.cvedetails.com/vulnerability-list/vendor_id-74/product_id-128/PHP-PHP.html'
    }
}

def main():
    parser = optparse.OptionParser()
    parser.add_option('-m', '--module', dest="module", default=None, help="The module to update")
    parser.add_option('-a', '--all', dest="all", default=False, action="store_true", help="Update all modules")
    parser.add_option('-v', dest="verbose", default=False, action="store_true", help="Show verbose stuff")
    options, args = parser.parse_args()
    logging.basicConfig(level=logging.DEBUG if options.verbose else logging.INFO)
    if options.all:
        for root in config:
            for entry in config[root]:
                print("Updating database %s_vulns.json with entries from %s" % (root, entry))
                process.CVEProcessor().run(config[root], root)

    elif options.module:
        module = options.module
        if module not in config:
            print("Unknown module %s, options are:" % module)
            for key in config:
                print(key)
            sys.exit(1)
        for entry in config[module]:
            print("Updating database %s_vulns.json with entries from %s" % (module, entry))
            process.CVEProcessor().run(config[module], module)
    else:
        parser.print_help()
        sys.exit(1)


if __name__ == "__main__":
    main()



import os
import logging
import sys

from helios.ext.libcms.detector import CMSDetector


# Main module loader for CMS Scanners
class CustomModuleLoader:
    folder = ""
    blacklist = ['cms_scanner.py', '__init__.py']
    modules = []
    logger = None
    is_aggressive = False
    module = None
    # common paths for cms installations
    cms_sub = ['wordpress', 'wp', 'magento', 'joomla', 'blog', 'drupal']

    def __init__(self, folder='scanners', blacklist=[], is_aggressive=False, log_level=logging.INFO):
        self.blacklist.extend(blacklist)
        self.folder = os.path.join(os.path.dirname(__file__), folder)
        self.logger = logging.getLogger("CMS Scanner")
        self.logger.setLevel(log_level)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(log_level)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.logger.addHandler(ch)
        self.logger.debug("Loading CMS scripts")
        self.is_aggressive = is_aggressive

    def load(self, script, name):
        sys.path.insert(0, os.path.dirname(__file__))
        base = script.replace('.py', '')
        try:
            command_module = __import__("scanners.%s" % base, fromlist=["scanners"])
            module = command_module.Scanner()
            if module.name == name:
                self.module = module
                self.module.set_logger(self.logger.getEffectiveLevel())
                self.logger.debug("Selected %s for target" % name)
        except ImportError as e:
            self.logger.warning("Error importing script:%s %s" % (base, str(e)))
        except Exception as e:
            self.logger.warning("Error loading script:%s %s" % (base, str(e)))

    def load_modules(self, name):
        for f in os.listdir(self.folder):
            if not f.endswith('.py'):
                continue
            if f in self.blacklist:
                continue
            self.load(f, name)

    def run_scripts(self, base, headers={}, cookies={}):
        p = CMSDetector()
        cms = p.scan(base)
        if cms:
            self.logger.debug("Detected %s as active CMS" % cms)
            self.load_modules(cms)
            if self.module:
                results = self.module.run(base)
                return {cms: results}
            else:
                self.logger.warning("No script was found for CMS %s" % cms)
        for cms_sub in self.cms_sub:
            cms = p.scan_sub(base, cms_sub)
            if cms:
                cms_url = "%s/%s/" % (base[:-1] if base.endswith('/') else base, cms_sub)
                self.logger.info("CMS %s was detected in folder %s" % (cms, cms_url))
                self.logger.debug("Detected %s as active CMS" % cms)
                self.load_modules(cms)
                if self.module:
                    results = self.module.run(cms_url)
                    return {cms: results}
                else:
                    self.logger.warning("No script was found for CMS %s" % cms)
                    return None


        else:
            self.logger.info("No CMS was detected on target %s" % base)
        return None




import os
import logging
import sys
import json


class CustomModuleLoader:
    folder = ""
    blacklist = ['module_base.py', '__init__.py']
    modules = []
    logger = None
    options = None
    writer = None
    scope = None
    sslverify = False
    headers = {}
    cookies = {}

    def __init__(self, folder='helios/modules', blacklist=[], options=None, logger=logging.INFO, database=None, scope=None):
        folder = os.path.join(os.path.dirname(__file__), '..', 'modules')
        self.blacklist.extend(blacklist)
        self.options = options
        self.folder = folder
        self.logger = logging.getLogger("CustomModuleLoader")
        self.logger.setLevel(logger)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logger)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        self.logger.addHandler(ch)
        self.logger.debug("Loading custom modules")
        self.scope = scope
        self.load_modules()
        self.writer = database

    def load(self, f):
        base = f.replace('.py', '')
        try:
            command_module = __import__("helios.modules.%s" % base, fromlist=["modules"])
            module = command_module.Module()
            if self.options:
                if 'all' not in self.options:
                    for opt in module.module_types:
                        if opt not in self.options:
                            self.logger.debug("Disabling module %s because %s is not enabled" % (module.name, opt))
                            return
            else:
                if 'dangerous' in module.module_types:
                    self.logger.debug(
                        "Disabling script %s because dangerous flag is present, "
                        "use --options all or add the dangerous flag to override" % (
                            module.name))
                    return
            module.scope = self.scope
            module.verify = self.sslverify
            module.headers = self.headers
            module.cookies = self.cookies
            self.modules.append(module)
            self.logger.debug("Enabled module: %s" % f)
        except ImportError as e:
            self.logger.warning("Error importing module:%s %s" % (f, str(e)))
        except Exception as e:
            self.logger.warning("Error loading module:%s %s" % (f, str(e)))

    def load_modules(self):
        for f in os.listdir(self.folder):
            if not f.endswith('.py'):
                continue
            if f in self.blacklist:
                continue
            self.load(f)

    def run_post(self, urltree, headers={}, cookies={}):
        output = []
        for module in self.modules:
            if 1:
                if module.input == "urldata":
                    for row in urltree:
                        try:
                            url, data = row
                            self.logger.debug("Running module %s on url: %s" % (module.name, url))
                            results = module.run(url, data, headers, cookies)
                            if results and len(results):
                                for r in results:
                                    if self.writer:
                                        self.writer.put(result_type="Module - Adv", script=module.name,
                                                    severity=module.severity, text=json.dumps(r))
                                    self.logger.info("Module %s Discovered %s" % (module.name, r))
                                    output.extend([module.name, r])
                        except Exception as e:
                            self.logger.warning("Error executing module %s on %s %s: %s" % (module.name, url, data, str(e)))
                if module.input == "urls":
                    self.logger.debug("Running module %s on %d urls" % (module.name, len(urltree)))
                    try:
                        results = module.run(urltree, headers, cookies)
                        if results and len(results):
                            for r in results:
                                self.logger.info("Module %s Discovered %s" % (module.name, r))
                                if self.writer:
                                    self.writer.put(result_type="Module - Adv", script=module.name,
                                                    severity=module.severity, text=json.dumps(r))
                                output.extend([module.name, r])
                    except Exception as e:
                        self.logger.warning("Error executing module %s on urls: %s" % (module.name, str(e)))

        return output

    def base_crawler(self, base):
        output = []
        for module in self.modules:
            if module.input == "base" and module.output == "crawler":
                self.logger.debug("Running pre-crawl module %s on base url %s" % (module.name, base))
                results = module.run(base)
                output.extend(results)
        return output



try:
    from urllib import quote_plus
except ImportError:
    from urllib.parse import quote_plus


class Base:
    module_types = []
    name = ""
    active = True
    # if module can send requests
    request = None
    should_run = True
    input = ""
    output = ""
    severity = 0
    verify = True
    scope = None
    headers = {

    }
    cookies = {

    }

    def run(self, url, data, headers, cookies):
        pass

    def run_passive(self, response):
        pass

    def close(self):
        print("Stopping script")
        self.should_run = False

    def get_params_from_url(self, url):
        if "?" not in url:
            return url, {}
        base, params = url.split('?')
        out = {}
        for sub in params.split('&'):
            parts = sub.split('=')
            if len(sub) > 1:
                out[parts[0]] = parts[1]
            else:
                out[parts[0]] = ""
        return base, out

    def params_to_url(self, base, params):
        out = []
        for key in params:
            out.append("%s=%s" % (quote_plus(key), quote_plus(params[key])))
        return base + "?" +"&".join(out)



from concurrent.futures import ThreadPoolExecutor
try:
    from Queue import Queue, Empty
except ImportError:
    from queue import Queue, Empty
import copy
import logging
import sys
import time

class Scanner:
    queue = None
    pool = None
    results = []
    thread_count = 10
    script_engine = None
    # bad idea since results are in list form
    copy_engine = False
    done = 0

    # multi-threading script engine
    def __init__(self, thread_count=None, script_engine=None, logger=logging.INFO):
        if thread_count:
            self.thread_count = thread_count
        self.script_engine = script_engine
        self.logger = logging.getLogger("Scanner")
        self.logger.setLevel(logger)
        ch = logging.StreamHandler(sys.stdout)
        ch.setLevel(logger)
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)
        if not self.logger.handlers:
            self.logger.addHandler(ch)
        self.logger.debug("Starting Scanner")
        self.queue = Queue()

    def run_scripts(self, request_object):
        scanner = None
        if self.copy_engine:
            scanner = copy.copy(self.script_engine)
        else:
            scanner = self.script_engine
        try:
            scanner.run_scripts(request_object)
        except Exception as e:
            self.logger.warning("Script error %s" % str(e))
        self.done += 1

    def run(self):
        self.pool = ThreadPoolExecutor(max_workers=self.thread_count)
        while True:
            try:
                request = self.queue.get(timeout=1)
                self.logger.debug("Running scripts on %s" % request.url)
                self.pool.submit(self.run_scripts, request)
            except Empty:
                break
            except Exception as e:
                self.logger.warning("Error: %s" % str(e))
                continue
            self.logger.debug("Todo: {0} Done: {1}".format(self.queue.qsize(), self.done))

        time.sleep(5)



from helios.webapp import base_app
import re
from helios.core.utils import requests_response_to_dict
import json
import requests
try:
    from urlparse import urljoin
except ImportError:
    from urllib.parse import urljoin


# This script detects vulnerabilities in the following PHP based products:
# - phpMyAdmin
class Scanner(base_app.BaseAPP):

    def __init__(self):
        self.name = "phpMyAdmin"
        self.types = []

    def detect(self, url):
        directories = ['phpmyadmin', 'pma', 'phpMyAdmin', '']
        for d in directories:
            path = urljoin(url, d)
            response = self.send(path)
            if response and response.status_code == 200 and "phpmyadmin.css.php" in response.text:
                self.app_url = path
                return True
        return False

    def test(self, url):
        docu_url = urljoin(url, "Documentation.html")
        docu_response = self.send(docu_url)
        version = None
        if docu_response:
            get_version = re.search(r'<title>phpMyAdmin\s([\d\.]+)\s', docu_response.text)
            if get_version:
                version = get_version.group(1)
                self.logger.info("phpMyAdmin version %s was identified from the Documentation.html file" % version)

        if version:
            db = self.get_db("phpmyadmin_vulns.json")
            data = json.loads(db)
            pma_vulns = data['phpMyAdmin']
            self.match_versions(pma_vulns, version, url)
        self.test_auth(url)

    def test_auth(self, url):
        sess = requests.session()
        default_creds = ['root:', 'root:admin', 'root:root']
        init_req = sess.get(url)
        if not init_req:
            self.logger.warning("Unable to test authentication, invalid initial response")
            return
        token_re = re.search('name="token".+?value="(.+?)"', init_req.text)

        for entry in default_creds:
            if not token_re:
                self.logger.warning("Unable to test authentication, no token")
                return
            user, passwd = entry.split(':')
            payload = {'lang': 'en', 'pma_username': user, 'pma_password': passwd, 'token': token_re.group(1)}
            post_url = urljoin(url, 'index.php')
            post_response = sess.post(post_url, payload)
            if post_response and 'Refresh' in post_response.headers:
                returl = post_response.headers['Refresh'].split(';')[1].strip()
                retdata = sess.get(returl)
                if retdata:
                    if 'class="loginform">' not in retdata.text:
                        match_str = "Possible positive authentication for user: %s and password %s on %s " % \
                                    (user, passwd, url)
                        result = {
                            'request': requests_response_to_dict(post_response),
                            'match': match_str
                        }
                        self.logger.info(match_str)
                        self.results.append(result)
                        return
                    else:
                        token_re = re.search('name="token".+?value="(.+?)"', retdata.text)









import random
import requests
import helios.modules.module_base
from helios.core.utils import requests_response_to_dict


class Module(helios.modules.module_base.Base):
    cookies = None
    headers = None

    def __init__(self):
        self.name = "Stored XSS"
        self.injections = {}
        self.module_types = ['injection', 'dangerous']
        self.possibilities = [
            '<script>var a = {injection_value};</script>',
            '<xss>var a = {injection_value};</xss>',
            '<img src="{injection_value}" onerror="" />'
        ]
        self.input = "urls"
        self.output = "vuln"
        self.severity = 3

    def run(self, urls, headers={}, cookies={}):
        self.cookies = cookies
        self.headers = headers
        for url in urls:
            url, data = url
            base, params = self.get_params_from_url(url)
            for u in params:
                self.inject(base, params, data, parameter_get=u)
            if data:
                for u in data:
                    self.inject(base, params, data, parameter_post=u)
        results = []
        for url in urls:
            url, data = url
            result = self.validate(url, data)
            if result:
                results.append(result)
        return results

    def validate(self, url, data):
        base, params = self.get_params_from_url(url)
        result = self.send(base, params, data)
        if result:
            for t in self.injections:
                for p in self.possibilities:
                    payload = p.replace('{injection_value}', str(t))
                    if payload in result.text:
                        return {'request': requests_response_to_dict(self.injections[t]), 'match': requests_response_to_dict(result)}
        return False

    def inject(self, url, params, data=None, parameter_get=None, parameter_post=None):
        if parameter_get:
            tmp = dict(params)
            for injection_value in self.possibilities:
                random_int = random.randint(9999, 9999999)
                payload = injection_value.replace('{injection_value}', str(random_int))
                payload = payload.replace('{original_value}', str(params[parameter_get]))
                tmp[parameter_get] = payload
                result = self.send(url, tmp, data)
                self.injections[str(random_int)] = result

        if parameter_post:
            tmp = dict(data)
            for injection_value in self.possibilities:
                random_int = random.randint(9999, 9999999)
                payload = injection_value.replace('{injection_value}', str(random_int))
                payload = payload.replace('{original_value}', str(data[parameter_post]))
                tmp[parameter_post] = payload
                result = self.send(url, params, tmp)
                self.injections[str(random_int)] = result

    def send(self, url, params, data):
        result = None
        headers = self.headers
        cookies = self.cookies
        try:
            if data:
                result = requests.post(url, params=params, data=data, headers=headers, cookies=cookies)
            else:
                result = requests.get(url, params=params, headers=headers, cookies=cookies)
        except Exception as e:
            print(str(e))
        return result
